/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : SCPMutationWD.h                                          */
/*                                                                       */
/*       Date : March 5, 1998                                            */
/* Updated at : March 5, 1998                                            */
/*                                                                       */
/*     Author : H. Timucin OZDEMIR                                       */
/*                                                                       */
/*************************************************************************/
/*************************************************************************/
/*  Mutation operator for Set Cover Problem

    1. Let w[i] denote the number of columns which covers row i
    2. For all i, let w[i] = w[i]-1
    3. Let sumw = sum_of(w{i]), i=1,...,m
    4. generate a random number (r) in range [1,sumw]
    5. Let k=0, sum=w[0]
    6. while( sum>r ) { k++; sum += w[k]; }
    7. Find indices of columns that cover row k
    8. Pick one randomly and delete
    9. restore the solution by abondaning the deleted column.
   10. reduce
*/
/*************************************************************************/

#ifndef _SCPMutationWD_h
#define _SCPMutationWD_h

#include <MutationOperator.h>
#include <Chromosomes.h>
#include <tSet.h>

#include "SCPChromosomes.h"

class SCPMutationWD : public MutationOperator
{
   public :

      SCPMutationWD();
      virtual ~SCPMutationWD();
      virtual Chromosomes * apply(Chromosomes *_or);
      virtual ChromosomesBag apply(Chromosomes *_or,Chromosomes *_fr,
                                     float percent);

   protected:
      virtual void repair(SCPChromosomes*,tSet<int>*);
      
}; // end of Class SCPMutationWD

#endif
